@@ -12,20 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import logging
-import sys
-from typing import Any, Dict, List, Optional, Set, Tuple, cast
+from typing import Any, Dict, List, Optional, Set, Tuple
 
 import requests
 import requests_toolbelt
-import tqdm
-import urllib3
-from requests import adapters
-from requests_toolbelt.utils import user_agent
+import rich.progress
+from rich import print
 
-import twine
 from twine import package as package_file
-
-KEYWORDS_TO_NOT_FLATTEN = {"gpg_signature", "content"}
+from twine.utils import make_requests_session
 
 LEGACY_PYPI = "https://pypi.python.org/"
 LEGACY_TEST_PYPI = "https://testpypi.python.org/"
@@ -37,16 +32,6 @@ WAREHOUSE_WEB = "https://pypi.org/"
 logger = logging.getLogger(__name__)
 
 
-class ProgressBar(tqdm.tqdm):
-    def update_to(self, n: int) -> None:
-        """Update the bar in the way compatible with requests-toolbelt.
-
-        This is identical to tqdm.update, except ``n`` will be the current
-        value - not the delta as tqdm expects.
-        """
-        self.update(n - self.n)  # will also do self.n = n
-
-
 class Repository:
     def __init__(
         self,
@@ -57,7 +42,7 @@ class Repository:
     ) -> None:
         self.url = repository_url
 
-        self.session = requests.session()
+        self.session = make_requests_session()
         # requests.Session.auth should be Union[None, Tuple[str, str], ...]
         # But username or password could be None
         # See TODO for utils.RepositoryConfig
@@ -67,57 +52,35 @@ class Repository:
         logger.info(f"username: {username if username else '<empty>'}")
         logger.info(f"password: <{'hidden' if password else 'empty'}>")
 
-        self.session.headers["User-Agent"] = self._make_user_agent_string()
-        for scheme in ("http://", "https://"):
-            self.session.mount(scheme, self._make_adapter_with_retries())
-
         # Working around https://github.com/python/typing/issues/182
         self._releases_json_data: Dict[str, Dict[str, Any]] = {}
         self.disable_progress_bar = disable_progress_bar
 
-    @staticmethod
-    def _make_adapter_with_retries() -> adapters.HTTPAdapter:
-        retry_kwargs = dict(
-            connect=5,
-            total=10,
-            status_forcelist=[500, 501, 502, 503],
-        )
-
-        try:
-            retry = urllib3.Retry(allowed_methods=["GET"], **retry_kwargs)
-        except TypeError:  # pragma: no cover
-            # Avoiding DeprecationWarning starting in urllib3 1.26
-            # Remove when that's the mininum version
-            retry = urllib3.Retry(method_whitelist=["GET"], **retry_kwargs)
-
-        return adapters.HTTPAdapter(max_retries=retry)
-
-    @staticmethod
-    def _make_user_agent_string() -> str:
-        from twine import cli
-
-        dependencies = cli.list_dependencies_and_versions()
-        user_agent_string = (
-            user_agent.UserAgentBuilder("twine", twine.__version__)
-            .include_extras(dependencies)
-            .include_implementation()
-            .build()
-        )
-
-        return cast(str, user_agent_string)
-
     def close(self) -> None:
         self.session.close()
 
     @staticmethod
-    def _convert_data_to_list_of_tuples(data: Dict[str, Any]) -> List[Tuple[str, Any]]:
-        data_to_send = []
+    def _convert_metadata_to_list_of_tuples(
+        data: package_file.PackageMetadata,
+    ) -> List[Tuple[str, Any]]:
+        # This does what ``warehouse.forklift.parse_form_metadata()`` does, in reverse.
+        data_to_send: List[Tuple[str, Any]] = []
         for key, value in data.items():
-            if key in KEYWORDS_TO_NOT_FLATTEN or not isinstance(value, (list, tuple)):
+            if key == "gpg_signature":
+                assert isinstance(value, tuple)
                 data_to_send.append((key, value))
+            elif key == "project_urls":
+                assert isinstance(value, dict)
+                for name, url in value.items():
+                    data_to_send.append((key, f"{name}, {url}"))
+            elif key == "keywords":
+                assert isinstance(value, list)
+                data_to_send.append((key, ", ".join(value)))
+            elif isinstance(value, (list, tuple)):
+                data_to_send.extend((key, item) for item in value)
             else:
-                for item in value:
-                    data_to_send.append((key, item))
+                assert isinstance(value, str)
+                data_to_send.append((key, value))
         return data_to_send
 
     def set_certificate_authority(self, cacert: Optional[str]) -> None:
@@ -129,12 +92,12 @@ class Repository:
             self.session.cert = clientcert
 
     def register(self, package: package_file.PackageFile) -> requests.Response:
-        data = package.metadata_dictionary()
-        data.update({":action": "submit", "protocol_version": "1"})
-
         print(f"Registering {package.basefilename}")
 
-        data_to_send = self._convert_data_to_list_of_tuples(data)
+        metadata = package.metadata_dictionary()
+        data_to_send = self._convert_metadata_to_list_of_tuples(metadata)
+        data_to_send.append((":action", "submit"))
+        data_to_send.append(("protocol_version", "1"))
         encoder = requests_toolbelt.MultipartEncoder(data_to_send)
         resp = self.session.post(
             self.url,
@@ -147,35 +110,42 @@ class Repository:
         return resp
 
     def _upload(self, package: package_file.PackageFile) -> requests.Response:
-        data = package.metadata_dictionary()
-        data.update(
-            {
-                # action
-                ":action": "file_upload",
-                "protocol_version": "1",
-            }
-        )
-
-        data_to_send = self._convert_data_to_list_of_tuples(data)
-
         print(f"Uploading {package.basefilename}")
 
+        metadata = package.metadata_dictionary()
+        data_to_send = self._convert_metadata_to_list_of_tuples(metadata)
+        data_to_send.append((":action", "file_upload"))
+        data_to_send.append(("protocol_version", "1"))
         with open(package.filename, "rb") as fp:
             data_to_send.append(
-                ("content", (package.basefilename, fp, "application/octet-stream"))
+                (
+                    "content",
+                    (package.basefilename, fp, "application/octet-stream"),
+                )
             )
             encoder = requests_toolbelt.MultipartEncoder(data_to_send)
-            with ProgressBar(
-                total=encoder.len,
-                unit="B",
-                unit_scale=True,
-                unit_divisor=1024,
-                miniters=1,
-                file=sys.stdout,
+
+            with rich.progress.Progress(
+                "[progress.percentage]{task.percentage:>3.0f}%",
+                rich.progress.BarColumn(),
+                rich.progress.DownloadColumn(),
+                "•",
+                rich.progress.TimeRemainingColumn(
+                    compact=True,
+                    elapsed_when_finished=True,
+                ),
+                "•",
+                rich.progress.TransferSpeedColumn(),
                 disable=self.disable_progress_bar,
-            ) as bar:
+            ) as progress:
+                task_id = progress.add_task("", total=encoder.len)
+
                 monitor = requests_toolbelt.MultipartEncoderMonitor(
-                    encoder, lambda monitor: bar.update_to(monitor.bytes_read)
+                    encoder,
+                    lambda monitor: progress.update(
+                        task_id,
+                        completed=monitor.bytes_read,
+                    ),
                 )
 
                 resp = self.session.post(
@@ -198,15 +168,10 @@ class Repository:
                 return resp
             if 500 <= resp.status_code < 600:
                 number_of_redirects += 1
-                print(
-                    'Received "{status_code}: {reason}" Package upload '
-                    "appears to have failed.  Retry {retry} of "
-                    "{max_redirects}".format(
-                        status_code=resp.status_code,
-                        reason=resp.reason,
-                        retry=number_of_redirects,
-                        max_redirects=max_redirects,
-                    )
+                logger.warning(
+                    f'Received "{resp.status_code}: {resp.reason}"'
+                    "\nPackage upload appears to have failed."
+                    f" Retry {number_of_redirects} of {max_redirects}."
                 )
             else:
                 return resp
@@ -228,7 +193,7 @@ class Repository:
             releases = self._releases_json_data.get(safe_name)
 
         if releases is None:
-            url = "{url}pypi/{package}/json".format(package=safe_name, url=LEGACY_PYPI)
+            url = f"{LEGACY_PYPI}pypi/{safe_name}/json"
             headers = {"Accept": "application/json"}
             response = self.session.get(url, headers=headers)
             if response.status_code == 200:
@@ -237,7 +202,7 @@ class Repository:
                 releases = {}
             self._releases_json_data[safe_name] = releases
 
-        packages = releases.get(package.metadata.version, [])
+        packages = releases.get(package.version, [])
 
         for uploaded_package in packages:
             if uploaded_package["filename"] == package.basefilename:
@@ -254,11 +219,11 @@ class Repository:
             return set()
 
         return {
-            "{}project/{}/{}/".format(url, package.safe_name, package.metadata.version)
+            f"{url}project/{package.safe_name}/{package.version}/"
             for package in packages
         }
 
     def verify_package_integrity(self, package: package_file.PackageFile) -> None:
         # TODO(sigmavirus24): Add a way for users to download the package and
-        # check it's hash against what it has locally.
+        # check its hash against what it has locally.
         pass
